@@ -4,26 +4,28 @@
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
- *     * Redistributions of source code must retain the above copyright
- *       notice, this list of conditions and the following disclaimer.
- *     * Redistributions in binary form must reproduce the above copyright
- *       notice, this list of conditions and the following disclaimer in the
- *       documentation and/or other materials provided with the distribution.
- *     * Neither the name "TwelveMonkeys" nor the
- *       names of its contributors may be used to endorse or promote products
- *       derived from this software without specific prior written permission.
  *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
- * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
- * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
- * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
- * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
- * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
- * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
- * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
- * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * * Redistributions of source code must retain the above copyright notice, this
+ *   list of conditions and the following disclaimer.
+ *
+ * * Redistributions in binary form must reproduce the above copyright notice,
+ *   this list of conditions and the following disclaimer in the documentation
+ *   and/or other materials provided with the distribution.
+ *
+ * * Neither the name of the copyright holder nor the names of its
+ *   contributors may be used to endorse or promote products derived from
+ *   this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+ * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
+ * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
+ * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
+ * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
 package com.twelvemonkeys.imageio;
@@ -70,12 +72,13 @@ public abstract class ImageReaderBase extends ImageReader {
     /**
      * Constructs an {@code ImageReader} and sets its
      * {@code originatingProvider} field to the supplied value.
-     * <p/>
-     * <p> Subclasses that make use of extensions should provide a
+     * <p>
+     * Subclasses that make use of extensions should provide a
      * constructor with signature {@code (ImageReaderSpi,
      * Object)} in order to retrieve the extension object.  If
      * the extension object is unsuitable, an
      * {@code IllegalArgumentException} should be thrown.
+     * </p>
      *
      * @param provider the {@code ImageReaderSpi} that is invoking this constructor, or {@code null}.
      */
@@ -203,9 +206,10 @@ public abstract class ImageReaderBase extends ImageReader {
 
     /**
      * Returns the {@code BufferedImage} to which decoded pixel data should be written.
-     * <p/>
+     * <p>
      * As {@link javax.imageio.ImageReader#getDestination} but tests if the explicit destination
      * image (if set) is valid according to the {@code ImageTypeSpecifier}s given in {@code types}.
+     * </p>
      *
      * @param param an {@code ImageReadParam} to be used to get
      * the destination image or image type, or {@code null}.
@@ -308,8 +312,14 @@ public abstract class ImageReaderBase extends ImageReader {
         int destWidth = destRegion.x + destRegion.width;
         int destHeight = destRegion.y + destRegion.height;
 
-        if ((long) destWidth * destHeight > Integer.MAX_VALUE) {
-            throw new IllegalArgumentException(String.format("destination width * height > Integer.MAX_VALUE: %d", (long) destWidth * destHeight));
+        long dimension = (long) destWidth * destHeight;
+        if (dimension > Integer.MAX_VALUE) {
+            throw new IllegalArgumentException(String.format("destination width * height > Integer.MAX_VALUE: %d", dimension));
+        }
+
+        long size = dimension * imageType.getSampleModel().getNumDataElements();
+        if (size > Integer.MAX_VALUE) {
+            throw new IllegalArgumentException(String.format("destination width * height * samplesPerPixel > Integer.MAX_VALUE: %d", size));
         }
 
         // Create a new image based on the type specifier
@@ -320,9 +330,10 @@ public abstract class ImageReaderBase extends ImageReader {
      * Utility method for getting the area of interest (AOI) of an image.
      * The AOI is defined by the {@link javax.imageio.IIOParam#setSourceRegion(java.awt.Rectangle)}
      * method.
-     * <p/>
+     * <p>
      * Note: If it is possible for the reader to read the AOI directly, such a
      * method should be used instead, for efficiency.
+     * </p>
      *
      * @param pImage the image to get AOI from
      * @param pParam the param optionally specifying the AOI
@@ -340,12 +351,14 @@ public abstract class ImageReaderBase extends ImageReader {
      * The subsampling is defined by the
      * {@link javax.imageio.IIOParam#setSourceSubsampling(int, int, int, int)}
      * method.
-     * <p/>
+     * <p>
      * NOTE: This method does not take the subsampling offsets into
      * consideration.
-     * <p/>
+     * </p>
+     * <p>
      * Note: If it is possible for the reader to subsample directly, such a
      * method should be used instead, for efficiency.
+     * </p>
      *
      * @param pImage the image to subsample
      * @param pParam the param optionally specifying subsampling
