@@ -4,26 +4,28 @@
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
- *  Redistributions of source code must retain the above copyright
- * notice, this list of conditions and the following disclaimer.
- *  Redistributions in binary form must reproduce the above copyright
- * notice, this list of conditions and the following disclaimer in the
- * documentation and/or other materials provided with the distribution.
- *  Neither the name "TwelveMonkeys" nor the
- * names of its contributors may be used to endorse or promote products
- * derived from this software without specific prior written permission.
  *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
- * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
- * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
- * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
- * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
- * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
- * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
- * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
- * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * * Redistributions of source code must retain the above copyright notice, this
+ *   list of conditions and the following disclaimer.
+ *
+ * * Redistributions in binary form must reproduce the above copyright notice,
+ *   this list of conditions and the following disclaimer in the documentation
+ *   and/or other materials provided with the distribution.
+ *
+ * * Neither the name of the copyright holder nor the names of its
+ *   contributors may be used to endorse or promote products derived from
+ *   this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+ * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
+ * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
+ * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
+ * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
 package com.twelvemonkeys.imageio.color;
@@ -40,16 +42,19 @@ import java.awt.color.ICC_Profile;
 import java.io.IOException;
 import java.io.InputStream;
 import java.lang.ref.WeakReference;
+import java.security.MessageDigest;
+import java.security.NoSuchAlgorithmException;
 import java.util.Arrays;
 import java.util.Map;
 import java.util.Properties;
 
 /**
  * A helper class for working with ICC color profiles and color spaces.
- * <p />
+ * <p>
  * Standard ICC color profiles are read from system-specific locations
  * for known operating systems.
- * <p />
+ * </p>
+ * <p>
  * Color profiles may be configured by placing a property-file
  * {@code com/twelvemonkeys/imageio/color/icc_profiles.properties}
  * on the classpath, specifying the full path to the profiles.
@@ -57,8 +62,10 @@ import java.util.Properties;
  * can be downloaded from
  * <a href="http://www.color.org/profiles2.xalter">ICC</a>,
  * <a href="http://www.adobe.com/downloads/">Adobe</a> or other places.
- * <p />
+ *  * </p>
+ * <p>
  * Example property file:
+ * </p>
  * <pre>
  * # icc_profiles.properties
  * ADOBE_RGB_1998=/path/to/Adobe RGB 1998.icc
@@ -70,6 +77,10 @@ import java.util.Properties;
  * @version $Id: ColorSpaces.java,v 1.0 24.01.11 17.51 haraldk Exp$
  */
 public final class ColorSpaces {
+    // TODO: Consider creating our own ICC profile class, which just wraps the byte array,
+    // for easier access and manipulation until creating a "real" ICC_Profile/ColorSpace.
+    // This will also let us work around the issues in the LCMS implementation.
+
     final static boolean DEBUG = "true".equalsIgnoreCase(System.getProperty("com.twelvemonkeys.imageio.color.debug"));
 
     /** We need special ICC profile handling for KCMS vs LCMS. Delegate to specific strategy. */
@@ -78,9 +89,11 @@ public final class ColorSpaces {
     // NOTE: java.awt.color.ColorSpace.CS_* uses 1000-1004, we'll use 5000+ to not interfere with future additions
 
     /** The Adobe RGB 1998 (or compatible) color space. Either read from disk or built-in. */
+    @SuppressWarnings("WeakerAccess")
     public static final int CS_ADOBE_RGB_1998 = 5000;
 
     /** A best-effort "generic" CMYK color space. Either read from disk or built-in. */
+    @SuppressWarnings("WeakerAccess")
     public static final int CS_GENERIC_CMYK = 5001;
 
     // Weak references to hold the color spaces while cached
@@ -90,13 +103,27 @@ public final class ColorSpaces {
     // Cache for the latest used color spaces
     private static final Map<Key, ICC_ColorSpace> cache = new LRUHashMap<>(10);
 
+    static {
+        try {
+            // Force invocation of ProfileDeferralMgr.activateProfiles() to avoid JDK-6986863
+            ICC_Profile.getInstance(ColorSpace.CS_sRGB).getData();
+        }
+        catch (Throwable disasters) {
+            System.err.println("ICC Color Profile not properly activated due to the exception below.");
+            System.err.println("Expect to see JDK-6986863 in action, and consider filing a bug report to your JRE provider.");
+
+            disasters.printStackTrace();
+        }
+    }
+
     private ColorSpaces() {}
 
     /**
      * Creates an ICC color space from the given ICC color profile.
-     * <p />
+     * <p>
      * For standard Java color spaces, the built-in instance is returned.
      * Otherwise, color spaces are looked up from cache and created on demand.
+     * </p>
      *
      * @param profile the ICC color profile. May not be {@code null}.
      * @return an ICC color space
@@ -106,34 +133,52 @@ public final class ColorSpaces {
     public static ICC_ColorSpace createColorSpace(final ICC_Profile profile) {
         Validate.notNull(profile, "profile");
 
-        byte[] profileHeader = profile.getData(ICC_Profile.icSigHead);
+        // Fix profile before lookup/create
+        profileCleaner.fixProfile(profile);
+
+        byte[] profileHeader = getProfileHeaderWithProfileId(profile);
 
         ICC_ColorSpace cs = getInternalCS(profile.getColorSpaceType(), profileHeader);
         if (cs != null) {
             return cs;
         }
 
-        // Special case for color profiles with rendering intent != 0, see isOffendingColorProfile method
-        // NOTE: Rendering intent is really a 4 byte value, but legal values are 0-3 (ICC1v42_2006_05_1.pdf, 7.2.15, p. 19)
-        if (profileHeader[ICC_Profile.icHdrRenderingIntent] != 0) {
-            profileHeader[ICC_Profile.icHdrRenderingIntent] = 0;
-
-            // Test again if this is an internal CS
-            cs = getInternalCS(profile.getColorSpaceType(), profileHeader);
-            if (cs != null) {
-                return cs;
-            }
-
-            // Fix profile before lookup/create
-            profileCleaner.fixProfile(profile, profileHeader);
-        }
-        else {
-            profileCleaner.fixProfile(profile, null);
-        }
-
         return getCachedOrCreateCS(profile, profileHeader);
     }
 
+    private static byte[] getProfileHeaderWithProfileId(final ICC_Profile profile) {
+        // Get *entire profile data*... :-/
+        byte[] data = profile.getData();
+
+        // Clear out preferred CMM, platform & creator, as these does not affect the profile in any way
+        // - LCMS updates CMM + creator to "lcms" and platform to current platform
+        // - KCMS keeps the values in the file...
+        Arrays.fill(data, ICC_Profile.icHdrCmmId, ICC_Profile.icHdrCmmId + 4, (byte) 0);
+        Arrays.fill(data, ICC_Profile.icHdrPlatform, ICC_Profile.icHdrPlatform + 4, (byte) 0);
+        // + Clear out rendering intent, as this may be updated by application
+        Arrays.fill(data, ICC_Profile.icHdrRenderingIntent, ICC_Profile.icHdrRenderingIntent + 4, (byte) 0);
+        Arrays.fill(data, ICC_Profile.icHdrCreator, ICC_Profile.icHdrCreator + 4, (byte) 0);
+
+        // Clear out any existing MD5, as it is no longer correct
+        Arrays.fill(data, ICC_Profile.icHdrProfileID, ICC_Profile.icHdrProfileID + 16, (byte) 0);
+
+        // Generate new MD5 and store in header
+        byte[] md5 = computeMD5(data);
+        System.arraycopy(md5, 0, data, ICC_Profile.icHdrProfileID, md5.length);
+
+        // ICC profile header is the first 128 bytes
+        return Arrays.copyOf(data, 128);
+    }
+
+    private static byte[] computeMD5(byte[] data) {
+        try {
+            return MessageDigest.getInstance("MD5").digest(data);
+        }
+        catch (NoSuchAlgorithmException e) {
+            throw new IllegalStateException("Missing MD5 MessageDigest");
+        }
+    }
+
     private static ICC_ColorSpace getInternalCS(final int profileCSType, final byte[] profileHeader) {
         if (profileCSType == ColorSpace.TYPE_RGB && Arrays.equals(profileHeader, sRGB.header)) {
             return (ICC_ColorSpace) ColorSpace.getInstance(ColorSpace.CS_sRGB);
@@ -163,9 +208,12 @@ public final class ColorSpaces {
             if (cs == null) {
                 cs = new ICC_ColorSpace(profile);
 
-                // Validate the color space, to avoid caching bad color spaces
-                // Will throw IllegalArgumentException or CMMException if the profile is bad
-                cs.fromRGB(new float[] {1f, 0f, 0f});
+                validateColorSpace(cs);
+
+                // On LCMS, validation *alters* the profile header, need to re-generate key
+                key = profileCleaner.validationAltersProfileHeader()
+                      ? new Key(getProfileHeaderWithProfileId(cs.getProfile()))
+                      : key;
 
                 cache.put(key, cs);
             }
@@ -174,6 +222,16 @@ public final class ColorSpaces {
         }
     }
 
+    private static void validateColorSpace(final ICC_ColorSpace cs) {
+        // Validate the color space, to avoid caching bad profiles/color spaces
+        // Will throw IllegalArgumentException or CMMException if the profile is bad
+        cs.fromRGB(new float[] {0.999f, 0.5f, 0.001f});
+
+        // This breaks *some times* after validation of bad profiles,
+        // we'll let it blow up early in this case
+        cs.getProfile().getData();
+    }
+
     /**
      * Tests whether an ICC color profile is equal to the default sRGB profile.
      *
@@ -186,16 +244,17 @@ public final class ColorSpaces {
     public static boolean isCS_sRGB(final ICC_Profile profile) {
         Validate.notNull(profile, "profile");
 
-        return profile.getColorSpaceType() == ColorSpace.TYPE_RGB && Arrays.equals(profile.getData(ICC_Profile.icSigHead), sRGB.header);
+        return profile.getColorSpaceType() == ColorSpace.TYPE_RGB && Arrays.equals(getProfileHeaderWithProfileId(profile), sRGB.header);
     }
 
     /**
      * Tests whether an ICC color profile is known to cause problems for {@link java.awt.image.ColorConvertOp}.
-     * <p />
+     * <p>
      * <em>
      * Note that this method only tests if a color conversion using this profile is known to fail.
      * There's no guarantee that the color conversion will succeed even if this method returns {@code false}.
      * </em>
+     * </p>
      *
      * @param profile the ICC color profile. May not be {@code null}.
      * @return {@code true} if known to be offending, {@code false} otherwise
@@ -214,19 +273,21 @@ public final class ColorSpaces {
 
         // This is particularly annoying, as the byte copying isn't really necessary,
         // except the getRenderingIntent method is package protected in java.awt.color
-        byte[] data = profile.getData(ICC_Profile.icSigHead);
+        byte[] header = profile.getData(ICC_Profile.icSigHead);
 
-        return data[ICC_Profile.icHdrRenderingIntent] != 0;
+        return header[ICC_Profile.icHdrRenderingIntent] != 0 || header[ICC_Profile.icHdrRenderingIntent + 1] != 0
+                || header[ICC_Profile.icHdrRenderingIntent + 2] != 0 || header[ICC_Profile.icHdrRenderingIntent + 3] > 3;
     }
 
     /**
      * Tests whether an ICC color profile is valid.
      * Invalid profiles are known to cause problems for {@link java.awt.image.ColorConvertOp}.
-     * <p />
+     * <p>
      * <em>
      * Note that this method only tests if a color conversion using this profile is known to fail.
      * There's no guarantee that the color conversion will succeed even if this method returns {@code false}.
      * </em>
+     * </p>
      *
      * @param profile the ICC color profile. May not be {@code null}.
      * @return {@code profile} if valid.
@@ -234,16 +295,19 @@ public final class ColorSpaces {
      * @throws java.awt.color.CMMException if {@code profile} is invalid.
      */
     public static ICC_Profile validateProfile(final ICC_Profile profile) {
-        createColorSpace(profile); // Creating a color space will fail if the profile is bad
+        // Fix profile before validation
+        profileCleaner.fixProfile(profile);
+        validateColorSpace(new ICC_ColorSpace(profile));
 
         return profile;
     }
 
     /**
      * Returns the color space specified by the given color space constant.
-     * <p />
+     * <p>
      * For standard Java color spaces, the built-in instance is returned.
      * Otherwise, color spaces are looked up from cache and created on demand.
+     * </p>
      *
      * @param colorSpace the color space constant.
      * @return the {@link ColorSpace} specified by the color space constant.
@@ -275,6 +339,10 @@ public final class ColorSpaces {
                             }
                         }
 
+                        if (profile.getColorSpaceType() != ColorSpace.TYPE_RGB) {
+                            throw new IllegalStateException("Configured AdobeRGB1998 profile is not TYPE_RGB");
+                        }
+
                         adobeRGB1998 = new WeakReference<>(profile);
                     }
                 }
@@ -298,6 +366,10 @@ public final class ColorSpaces {
                             return CMYKColorSpace.getInstance();
                         }
 
+                        if (profile.getColorSpaceType() != ColorSpace.TYPE_CMYK) {
+                            throw new IllegalStateException("Configured Generic CMYK profile is not TYPE_CMYK");
+                        }
+
                         genericCMYK = new WeakReference<>(profile);
                     }
                 }
@@ -310,6 +382,7 @@ public final class ColorSpaces {
         }
     }
 
+    @SuppressWarnings("SameParameterValue")
     private static ICC_Profile readProfileFromClasspathResource(final String profilePath) {
         InputStream stream = ColorSpaces.class.getResourceAsStream(profilePath);
 
@@ -356,7 +429,7 @@ public final class ColorSpaces {
     private static final class Key {
         private final byte[] data;
 
-        public Key(byte[] data) {
+        Key(byte[] data) {
             this.data = data;
         }
 
@@ -369,30 +442,36 @@ public final class ColorSpaces {
         public int hashCode() {
             return Arrays.hashCode(data);
         }
+
+        @Override
+        public String toString() {
+            return getClass().getSimpleName() + "@" + Integer.toHexString(hashCode());
+        }
     }
 
     // Cache header profile data to avoid excessive array creation/copying in static inner class for on-demand lazy init
     private static class sRGB {
-        private static final byte[] header = ICC_Profile.getInstance(ColorSpace.CS_sRGB).getData(ICC_Profile.icSigHead);
+        private static final byte[] header = getProfileHeaderWithProfileId(ICC_Profile.getInstance(ColorSpace.CS_sRGB));
     }
 
     private static class CIEXYZ {
-        private static final byte[] header = ICC_Profile.getInstance(ColorSpace.CS_CIEXYZ).getData(ICC_Profile.icSigHead);
+        private static final byte[] header = getProfileHeaderWithProfileId(ICC_Profile.getInstance(ColorSpace.CS_CIEXYZ));
     }
 
     private static class PYCC {
-        private static final byte[] header = ICC_Profile.getInstance(ColorSpace.CS_PYCC).getData(ICC_Profile.icSigHead);
+        private static final byte[] header = getProfileHeaderWithProfileId(ICC_Profile.getInstance(ColorSpace.CS_PYCC));
     }
 
     private static class GRAY {
-        private static final byte[] header = ICC_Profile.getInstance(ColorSpace.CS_GRAY).getData(ICC_Profile.icSigHead);
+        private static final byte[] header = getProfileHeaderWithProfileId(ICC_Profile.getInstance(ColorSpace.CS_GRAY));
     }
 
     private static class LINEAR_RGB {
-        private static final byte[] header = ICC_Profile.getInstance(ColorSpace.CS_LINEAR_RGB).getData(ICC_Profile.icSigHead);
+        private static final byte[] header = getProfileHeaderWithProfileId(ICC_Profile.getInstance(ColorSpace.CS_LINEAR_RGB));
     }
 
     private static class Profiles {
+        // TODO: Honour java.iccprofile.path property?
         private static final Properties PROFILES = loadProfiles();
 
         private static Properties loadProfiles() {
@@ -439,7 +518,7 @@ public final class ColorSpaces {
             return profiles;
         }
 
-        public static String getPath(final String profileName) {
+        static String getPath(final String profileName) {
             return PROFILES.getProperty(profileName);
         }
     }
