@@ -4,38 +4,40 @@
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
- *     * Redistributions of source code must retain the above copyright
- *       notice, this list of conditions and the following disclaimer.
- *     * Redistributions in binary form must reproduce the above copyright
- *       notice, this list of conditions and the following disclaimer in the
- *       documentation and/or other materials provided with the distribution.
- *     * Neither the name "TwelveMonkeys" nor the
- *       names of its contributors may be used to endorse or promote products
- *       derived from this software without specific prior written permission.
  *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
- * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
- * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
- * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
- * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
- * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
- * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
- * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
- * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * * Redistributions of source code must retain the above copyright notice, this
+ *   list of conditions and the following disclaimer.
+ *
+ * * Redistributions in binary form must reproduce the above copyright notice,
+ *   this list of conditions and the following disclaimer in the documentation
+ *   and/or other materials provided with the distribution.
+ *
+ * * Neither the name of the copyright holder nor the names of its
+ *   contributors may be used to endorse or promote products derived from
+ *   this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+ * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
+ * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
+ * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
+ * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
 package com.twelvemonkeys.imageio.plugins.tiff;
 
 import com.twelvemonkeys.image.ImageUtil;
 import com.twelvemonkeys.imageio.ImageWriterBase;
-import com.twelvemonkeys.imageio.metadata.AbstractEntry;
 import com.twelvemonkeys.imageio.metadata.Directory;
 import com.twelvemonkeys.imageio.metadata.Entry;
-import com.twelvemonkeys.imageio.metadata.exif.EXIFWriter;
-import com.twelvemonkeys.imageio.metadata.exif.Rational;
-import com.twelvemonkeys.imageio.metadata.exif.TIFF;
+import com.twelvemonkeys.imageio.metadata.tiff.Rational;
+import com.twelvemonkeys.imageio.metadata.tiff.TIFF;
+import com.twelvemonkeys.imageio.metadata.tiff.TIFFEntry;
+import com.twelvemonkeys.imageio.metadata.tiff.TIFFWriter;
 import com.twelvemonkeys.imageio.stream.SubImageOutputStream;
 import com.twelvemonkeys.imageio.util.IIOUtil;
 import com.twelvemonkeys.imageio.util.ProgressListenerBase;
@@ -56,12 +58,13 @@ import java.awt.color.ColorSpace;
 import java.awt.color.ICC_ColorSpace;
 import java.awt.image.*;
 import java.io.*;
-import java.lang.reflect.Array;
 import java.nio.ByteBuffer;
 import java.util.*;
 import java.util.zip.Deflater;
 import java.util.zip.DeflaterOutputStream;
 
+import static com.twelvemonkeys.imageio.plugins.tiff.TIFFStreamMetadata.configureStreamByteOrder;
+
 /**
  * TIFFImageWriter
  *
@@ -111,7 +114,7 @@ public final class TIFFImageWriter extends ImageWriterBase {
     /**
      * Metadata writer for sequence writing
      */
-    private EXIFWriter sequenceExifWriter = null;
+    private TIFFWriter sequenceTIFFWriter = null;
 
     /**
      * Position of last IFD Pointer on active sequence writing
@@ -129,97 +132,27 @@ public final class TIFFImageWriter extends ImageWriterBase {
         // TODO: Allow appending/partly overwrite of existing file...
     }
 
-    static final class TIFFEntry extends AbstractEntry {
-        // TODO: Expose a merge of this and the EXIFEntry class...
-        private final short type;
-
-        private static short guessType(final Object val) {
-            // TODO: This code is duplicated in EXIFWriter.getType, needs refactor!
-            Object value = Validate.notNull(val);
-
-            boolean array = value.getClass().isArray();
-            if (array) {
-                value = Array.get(value, 0);
-            }
-
-            // Note: This "narrowing" is to keep data consistent between read/write.
-            // TODO: Check for negative values and use signed types?
-            if (value instanceof Byte) {
-                return TIFF.TYPE_BYTE;
-            }
-            if (value instanceof Short) {
-                if (!array && (Short) value < Byte.MAX_VALUE) {
-                    return TIFF.TYPE_BYTE;
-                }
-
-                return TIFF.TYPE_SHORT;
-            }
-            if (value instanceof Integer) {
-                if (!array && (Integer) value < Short.MAX_VALUE) {
-                    return TIFF.TYPE_SHORT;
-                }
-
-                return TIFF.TYPE_LONG;
-            }
-            if (value instanceof Long) {
-                if (!array && (Long) value < Integer.MAX_VALUE) {
-                    return TIFF.TYPE_LONG;
-                }
-            }
-
-            if (value instanceof Rational) {
-                return TIFF.TYPE_RATIONAL;
-            }
-
-            if (value instanceof String) {
-                return TIFF.TYPE_ASCII;
-            }
-
-            // TODO: More types
-
-            throw new UnsupportedOperationException(String.format("Method guessType not implemented for value of type %s", value.getClass()));
-        }
-
-        TIFFEntry(final int identifier, final Object value) {
-            this(identifier, guessType(value), value);
-        }
-
-        TIFFEntry(int identifier, short type, Object value) {
-            super(identifier, value);
-            this.type = type;
-        }
-
-        @Override
-        public String getTypeName() {
-            return TIFF.TYPE_NAMES[type];
-        }
-    }
-
     @Override
     public void write(final IIOMetadata streamMetadata, final IIOImage image, final ImageWriteParam param) throws IOException {
-        // TODO: Validate input
-        assertOutput();
-        // TODO: streamMetadata?
-
-        // TODO: Make TIFFEntry and possibly TIFFDirectory? public
-        EXIFWriter exifWriter = new EXIFWriter();
-        exifWriter.writeTIFFHeader(imageOutput);
-
-        writePage(0, image, param, exifWriter, imageOutput.getStreamPosition());
-
-        imageOutput.flush();
+        prepareWriteSequence(streamMetadata);
+        writeToSequence(image, param);
+        endWriteSequence();
     }
 
-    private long writePage(int imageIndex, IIOImage image, ImageWriteParam param, EXIFWriter exifWriter, long lastIFDPointerOffset)
+    private long writePage(int imageIndex, IIOImage image, ImageWriteParam param, TIFFWriter tiffWriter, long lastIFDPointerOffset)
             throws IOException {
         RenderedImage renderedImage = image.getRenderedImage();
-
-        TIFFImageMetadata metadata = image.getMetadata() != null
-                                     ? convertImageMetadata(image.getMetadata(), ImageTypeSpecifier.createFromRenderedImage(renderedImage), param)
-                                     : getDefaultImageMetadata(ImageTypeSpecifier.createFromRenderedImage(renderedImage), param);
-
-        ColorModel colorModel = renderedImage.getColorModel();
         SampleModel sampleModel = renderedImage.getSampleModel();
+
+        // Can't use createFromRenderedImage in this case, as it does not consider palette for TYPE_BYTE_BINARY...
+        // TODO: Consider writing workaround in ImageTypeSpecifiers
+        ImageTypeSpecifier spec = new ImageTypeSpecifier(renderedImage);
+
+        // TODO: Handle case where convertImageMetadata returns null, due to unknown metadata format, or reconsider if that's a valid case...
+        TIFFImageMetadata metadata = image.getMetadata() != null
+                                     ? convertImageMetadata(image.getMetadata(), spec, param)
+                                     : getDefaultImageMetadata(spec, param);
+
         int numBands = sampleModel.getNumBands();
         int pixelSize = computePixelSize(sampleModel);
 
@@ -241,157 +174,41 @@ public final class TIFFImageWriter extends ImageWriterBase {
             throw new IllegalArgumentException("Unknown bit/bandOffsets for sample model: " + sampleModel);
         }
 
-        // TODO: There shouldn't be necessary to create a separate map here, this should be handled in the
-        // convertImageMetadata/getDefaultImageMetadata methods....
         Map<Integer, Entry> entries = new LinkedHashMap<>();
+        // Copy metadata to output
+        Directory metadataIFD = metadata.getIFD();
+        for (Entry entry : metadataIFD) {
+            entries.put((Integer) entry.getIdentifier(), entry);
+        }
+
         entries.put(TIFF.TAG_IMAGE_WIDTH, new TIFFEntry(TIFF.TAG_IMAGE_WIDTH, renderedImage.getWidth()));
         entries.put(TIFF.TAG_IMAGE_HEIGHT, new TIFFEntry(TIFF.TAG_IMAGE_HEIGHT, renderedImage.getHeight()));
-        entries.put(TIFF.TAG_ORIENTATION, new TIFFEntry(TIFF.TAG_ORIENTATION, 1)); // (optional)
-        entries.put(TIFF.TAG_BITS_PER_SAMPLE, new TIFFEntry(TIFF.TAG_BITS_PER_SAMPLE, asShortArray(sampleModel.getSampleSize())));
-
-        // If numComponents > numColorComponents, write ExtraSamples
-        if (numBands > colorModel.getNumColorComponents()) {
-            // TODO: Write per component > numColorComponents
-            if (colorModel.hasAlpha()) {
-                entries.put(TIFF.TAG_EXTRA_SAMPLES, new TIFFEntry(TIFF.TAG_EXTRA_SAMPLES, colorModel.isAlphaPremultiplied() ? TIFFBaseline.EXTRASAMPLE_ASSOCIATED_ALPHA : TIFFBaseline.EXTRASAMPLE_UNASSOCIATED_ALPHA));
-            }
-            else {
-                entries.put(TIFF.TAG_EXTRA_SAMPLES, new TIFFEntry(TIFF.TAG_EXTRA_SAMPLES, TIFFBaseline.EXTRASAMPLE_UNSPECIFIED));
-            }
-        }
-
-        // Write compression field from param or metadata
-        int compression;
-        if ((param == null || param.getCompressionMode() == TIFFImageWriteParam.MODE_COPY_FROM_METADATA)
-                && image.getMetadata() != null && metadata.getIFD().getEntryById(TIFF.TAG_COMPRESSION) != null) {
-            compression = ((Number) metadata.getIFD().getEntryById(TIFF.TAG_COMPRESSION).getValue()).intValue();
-        }
-        else {
-            compression = TIFFImageWriteParam.getCompressionType(param);
-        }
-
-        entries.put(TIFF.TAG_COMPRESSION, new TIFFEntry(TIFF.TAG_COMPRESSION, compression));
-
-        // TODO: Let param/metadata control predictor
-        // TODO: Depending on param.getCompressionMode(): DISABLED/EXPLICIT/COPY_FROM_METADATA/DEFAULT
-        switch (compression) {
-            case TIFFExtension.COMPRESSION_ZLIB:
-            case TIFFExtension.COMPRESSION_DEFLATE:
-            case TIFFExtension.COMPRESSION_LZW:
-                if (pixelSize >= 8) {
-                    entries.put(TIFF.TAG_PREDICTOR, new TIFFEntry(TIFF.TAG_PREDICTOR, TIFFExtension.PREDICTOR_HORIZONTAL_DIFFERENCING));
-                }
-
-                break;
-
-            case TIFFExtension.COMPRESSION_CCITT_T4:
-                Entry group3options = metadata.getIFD().getEntryById(TIFF.TAG_GROUP3OPTIONS);
-
-                if (group3options == null) {
-                    group3options = new TIFFEntry(TIFF.TAG_GROUP3OPTIONS, (long) TIFFExtension.GROUP3OPT_2DENCODING);
-                }
-
-                entries.put(TIFF.TAG_GROUP3OPTIONS, group3options);
-
-                break;
-
-            case TIFFExtension.COMPRESSION_CCITT_T6:
-                Entry group4options = metadata.getIFD().getEntryById(TIFF.TAG_GROUP4OPTIONS);
-
-                if (group4options == null) {
-                    group4options = new TIFFEntry(TIFF.TAG_GROUP4OPTIONS, 0L);
-                }
-
-                entries.put(TIFF.TAG_GROUP4OPTIONS, group4options);
-
-                break;
-
-            default:
-        }
-
-        int photometric = getPhotometricInterpretation(colorModel, compression);
-        entries.put(TIFF.TAG_PHOTOMETRIC_INTERPRETATION, new TIFFEntry(TIFF.TAG_PHOTOMETRIC_INTERPRETATION, photometric));
-
-        if (photometric == TIFFBaseline.PHOTOMETRIC_PALETTE && colorModel instanceof IndexColorModel) {
-            // TODO: Fix consistency between sampleModel.getSampleSize() and colorModel.getPixelSize()...
-            // We should be able to support 1, 2, 4 and 8 bits per sample at least, and probably 3, 5, 6 and 7 too
-            entries.put(TIFF.TAG_COLOR_MAP, new TIFFEntry(TIFF.TAG_COLOR_MAP, createColorMap((IndexColorModel) colorModel, sampleModel.getSampleSize(0))));
-            entries.put(TIFF.TAG_SAMPLES_PER_PIXEL, new TIFFEntry(TIFF.TAG_SAMPLES_PER_PIXEL, 1));
-        }
-        else {
-            entries.put(TIFF.TAG_SAMPLES_PER_PIXEL, new TIFFEntry(TIFF.TAG_SAMPLES_PER_PIXEL, numBands));
-
-            // Note: Assuming sRGB to be the default RGB interpretation
-            ColorSpace colorSpace = colorModel.getColorSpace();
-            if (colorSpace instanceof ICC_ColorSpace && !colorSpace.isCS_sRGB()) {
-                entries.put(TIFF.TAG_ICC_PROFILE, new TIFFEntry(TIFF.TAG_ICC_PROFILE, ((ICC_ColorSpace) colorSpace).getProfile().getData()));
-            }
-        }
-
-        // Default sample format SAMPLEFORMAT_UINT need not be written
-        if (sampleModel.getDataType() == DataBuffer.TYPE_SHORT/* TODO: if isSigned(sampleModel.getDataType) or getSampleFormat(sampleModel) != 0 */) {
-            entries.put(TIFF.TAG_SAMPLE_FORMAT, new TIFFEntry(TIFF.TAG_SAMPLE_FORMAT, TIFFExtension.SAMPLEFORMAT_INT));
-        }
-        // TODO: Float values!
-
-        // TODO: Again, this should be handled in the metadata conversion....
-        // Get Software from metadata, or use default
-        Entry software = metadata.getIFD().getEntryById(TIFF.TAG_SOFTWARE);
-        entries.put(TIFF.TAG_SOFTWARE, software != null ? software : new TIFFEntry(TIFF.TAG_SOFTWARE, "TwelveMonkeys ImageIO TIFF writer " + originatingProvider.getVersion()));
-
-        // Copy metadata to output
-        int[] copyTags = {
-                TIFF.TAG_ORIENTATION,
-                TIFF.TAG_DATE_TIME,
-                TIFF.TAG_DOCUMENT_NAME,
-                TIFF.TAG_IMAGE_DESCRIPTION,
-                TIFF.TAG_MAKE,
-                TIFF.TAG_MODEL,
-                TIFF.TAG_PAGE_NAME,
-                TIFF.TAG_PAGE_NUMBER,
-                TIFF.TAG_ARTIST,
-                TIFF.TAG_HOST_COMPUTER,
-                TIFF.TAG_COPYRIGHT
-        };
-        for (int tagID : copyTags) {
-            Entry entry = metadata.getIFD().getEntryById(tagID);
-            if (entry != null) {
-                entries.put(tagID, entry);
-            }
-        }
-
-        // Get X/YResolution and ResolutionUnit from metadata if set, otherwise use defaults
-        // TODO: Add logic here OR in metadata merging, to make sure these 3 values are consistent.
-        Entry xRes = metadata.getIFD().getEntryById(TIFF.TAG_X_RESOLUTION);
-        entries.put(TIFF.TAG_X_RESOLUTION, xRes != null ? xRes : new TIFFEntry(TIFF.TAG_X_RESOLUTION, STANDARD_DPI));
-        Entry yRes = metadata.getIFD().getEntryById(TIFF.TAG_Y_RESOLUTION);
-        entries.put(TIFF.TAG_Y_RESOLUTION, yRes != null ? yRes : new TIFFEntry(TIFF.TAG_Y_RESOLUTION, STANDARD_DPI));
-        Entry resUnit = metadata.getIFD().getEntryById(TIFF.TAG_RESOLUTION_UNIT);
-        entries.put(TIFF.TAG_RESOLUTION_UNIT, resUnit != null ? resUnit : new TIFFEntry(TIFF.TAG_RESOLUTION_UNIT, TIFFBaseline.RESOLUTION_UNIT_DPI));
 
         // TODO: RowsPerStrip - can be entire image (or even 2^32 -1), but it's recommended to write "about 8K bytes" per strip
         entries.put(TIFF.TAG_ROWS_PER_STRIP, new TIFFEntry(TIFF.TAG_ROWS_PER_STRIP, renderedImage.getHeight()));
-        // - StripByteCounts - for no compression, entire image data... (TODO: How to know the byte counts prior to writing data?)
+        // StripByteCounts - for no compression, entire image data...
         entries.put(TIFF.TAG_STRIP_BYTE_COUNTS, new TIFFEntry(TIFF.TAG_STRIP_BYTE_COUNTS, -1)); // Updated later
-        // - StripOffsets - can be offset to single strip only (TODO: but how large is the IFD data...???)
+        // StripOffsets - can be offset to single strip only
         entries.put(TIFF.TAG_STRIP_OFFSETS, new TIFFEntry(TIFF.TAG_STRIP_OFFSETS, -1)); // Updated later
 
         // TODO: If tiled, write tile indexes etc
         // Depending on param.getTilingMode
         long nextIFDPointerOffset = -1;
 
+        int compression = ((Number) entries.get(TIFF.TAG_COMPRESSION).getValue()).intValue();
+
         if (compression == TIFFBaseline.COMPRESSION_NONE) {
             // This implementation, allows semi-streaming-compatible uncompressed TIFFs
             long streamPosition = imageOutput.getStreamPosition();
 
-            long ifdSize = exifWriter.computeIFDSize(entries.values());
-            long stripOffset = streamPosition + 4 +  ifdSize + 4;
-            long stripByteCount = (renderedImage.getWidth() * renderedImage.getHeight() * pixelSize + 7) / 8;
+            long ifdSize = tiffWriter.computeIFDSize(entries.values());
+            long stripOffset = streamPosition + 4 + ifdSize + 4;
+            long stripByteCount = ((long) renderedImage.getWidth() * renderedImage.getHeight() * pixelSize + 7L) / 8L;
 
-            entries.put(TIFF.TAG_STRIP_OFFSETS, new TIFFEntry(TIFF.TAG_STRIP_OFFSETS, stripOffset));
-            entries.put(TIFF.TAG_STRIP_BYTE_COUNTS, new TIFFEntry(TIFF.TAG_STRIP_BYTE_COUNTS, stripByteCount));
+            entries.put(TIFF.TAG_STRIP_OFFSETS, new TIFFEntry(TIFF.TAG_STRIP_OFFSETS, TIFF.TYPE_LONG, stripOffset));
+            entries.put(TIFF.TAG_STRIP_BYTE_COUNTS, new TIFFEntry(TIFF.TAG_STRIP_BYTE_COUNTS, TIFF.TYPE_LONG, stripByteCount));
 
-            long ifdPointer = exifWriter.writeIFD(entries.values(), imageOutput); // NOTE: Writer takes case of ordering tags
+            long ifdPointer = tiffWriter.writeIFD(entries.values(), imageOutput); // NOTE: Writer takes case of ordering tags
             nextIFDPointerOffset = imageOutput.getStreamPosition();
 
             // If we have a previous IFD, update pointer
@@ -425,7 +242,7 @@ public final class TIFFImageWriter extends ImageWriterBase {
                 ListenerDelegate listener = new ListenerDelegate(imageIndex);
                 jpegWriter.addIIOWriteProgressListener(listener);
                 jpegWriter.addIIOWriteWarningListener(listener);
-                jpegWriter.write(renderedImage);
+                jpegWriter.write(null, image, copyParams(param, jpegWriter));
             }
             finally {
                 jpegWriter.dispose();
@@ -440,10 +257,10 @@ public final class TIFFImageWriter extends ImageWriterBase {
 
         // Update IFD0-pointer, and write IFD
         if (compression != TIFFBaseline.COMPRESSION_NONE) {
-            entries.put(TIFF.TAG_STRIP_OFFSETS, new TIFFEntry(TIFF.TAG_STRIP_OFFSETS, stripOffset));
-            entries.put(TIFF.TAG_STRIP_BYTE_COUNTS, new TIFFEntry(TIFF.TAG_STRIP_BYTE_COUNTS, stripByteCount));
+            entries.put(TIFF.TAG_STRIP_OFFSETS, new TIFFEntry(TIFF.TAG_STRIP_OFFSETS, TIFF.TYPE_LONG, stripOffset));
+            entries.put(TIFF.TAG_STRIP_BYTE_COUNTS, new TIFFEntry(TIFF.TAG_STRIP_BYTE_COUNTS, TIFF.TYPE_LONG, stripByteCount));
 
-            long ifdPointer = exifWriter.writeIFD(entries.values(), imageOutput); // NOTE: Writer takes case of ordering tags
+            long ifdPointer = tiffWriter.writeIFD(entries.values(), imageOutput); // NOTE: Writer takes case of ordering tags
 
             nextIFDPointerOffset = imageOutput.getStreamPosition();
 
@@ -460,6 +277,27 @@ public final class TIFFImageWriter extends ImageWriterBase {
         return nextIFDPointerOffset;
     }
 
+    // TODO: Candidate util method
+    private ImageWriteParam copyParams(final ImageWriteParam param, final ImageWriter writer) {
+        if (param == null) {
+            return null;
+        }
+
+        // Always safe
+        ImageWriteParam writeParam = writer.getDefaultWriteParam();
+        writeParam.setSourceSubsampling(param.getSourceXSubsampling(), param.getSourceYSubsampling(), param.getSubsamplingXOffset(), param.getSubsamplingYOffset());
+        writeParam.setSourceRegion(param.getSourceRegion());
+        writeParam.setSourceBands(param.getSourceBands());
+
+        // Only if canWriteCompressed()
+        writeParam.setCompressionMode(param.getCompressionMode());
+        if (param.getCompressionMode() == ImageWriteParam.MODE_EXPLICIT) {
+            writeParam.setCompressionQuality(param.getCompressionQuality());
+        }
+
+        return writeParam;
+    }
+
     // TODO: Candidate util method
     private int computePixelSize(final SampleModel sampleModel) {
         int size = 0;
@@ -581,6 +419,10 @@ public final class TIFFImageWriter extends ImageWriterBase {
             case TIFFBaseline.COMPRESSION_CCITT_MODIFIED_HUFFMAN_RLE:
             case TIFFExtension.COMPRESSION_CCITT_T4:
             case TIFFExtension.COMPRESSION_CCITT_T6:
+                if (image.getSampleModel().getNumBands() != 1 || image.getSampleModel().getSampleSize(0) != 1) {
+                    throw new IllegalArgumentException("CCITT compressions supports 1 sample/pixel, 1 bit/sample only");
+                }
+
                 long option = 0L;
 
                 if (compression != TIFFBaseline.COMPRESSION_CCITT_MODIFIED_HUFFMAN_RLE) {
@@ -907,7 +749,7 @@ public final class TIFFImageWriter extends ImageWriterBase {
                 }
             }
             catch (IIOInvalidTreeException e) {
-                // TODO: How to issue warning when warning requires imageIndex??? Use -1?
+                processWarningOccurred(sequenceIndex, "Could not convert image meta data: " + e.getMessage());
             }
 
             ifd = outData.getIFD();
@@ -922,12 +764,58 @@ public final class TIFFImageWriter extends ImageWriterBase {
 
         Map<Integer, Entry> entries = new LinkedHashMap<>(ifd != null ? ifd.size() + 10 : 20);
 
+        // Set software as default, may be overwritten
+        entries.put(TIFF.TAG_SOFTWARE, new TIFFEntry(TIFF.TAG_SOFTWARE, "TwelveMonkeys ImageIO TIFF writer " + originatingProvider.getVersion()));
+        entries.put(TIFF.TAG_ORIENTATION, new TIFFEntry(TIFF.TAG_ORIENTATION, 1)); // (optional)
+
         if (ifd != null) {
             for (Entry entry : ifd) {
-                entries.put((Integer) entry.getIdentifier(), entry);
+                int tagId = (Integer) entry.getIdentifier();
+
+                switch (tagId) {
+                    // Baseline
+                    case TIFF.TAG_SUBFILE_TYPE:
+                    case TIFF.TAG_OLD_SUBFILE_TYPE:
+                    case TIFF.TAG_IMAGE_DESCRIPTION:
+                    case TIFF.TAG_MAKE:
+                    case TIFF.TAG_MODEL:
+                    case TIFF.TAG_ORIENTATION:
+                    case TIFF.TAG_X_RESOLUTION:
+                    case TIFF.TAG_Y_RESOLUTION:
+                    case TIFF.TAG_RESOLUTION_UNIT:
+                    case TIFF.TAG_SOFTWARE:
+                    case TIFF.TAG_DATE_TIME:
+                    case TIFF.TAG_ARTIST:
+                    case TIFF.TAG_HOST_COMPUTER:
+                    case TIFF.TAG_COPYRIGHT:
+                    // Extension
+                    case TIFF.TAG_DOCUMENT_NAME:
+                    case TIFF.TAG_PAGE_NAME:
+                    case TIFF.TAG_X_POSITION:
+                    case TIFF.TAG_Y_POSITION:
+                    case TIFF.TAG_PAGE_NUMBER:
+                    case TIFF.TAG_XMP:
+                    // Private/Custom
+                    case TIFF.TAG_IPTC:
+                    case TIFF.TAG_PHOTOSHOP:
+                    case TIFF.TAG_PHOTOSHOP_IMAGE_SOURCE_DATA:
+                    case TIFF.TAG_PHOTOSHOP_ANNOTATIONS:
+                    case TIFF.TAG_EXIF_IFD:
+                    case TIFF.TAG_GPS_IFD:
+                    case TIFF.TAG_INTEROP_IFD:
+                        entries.put(tagId, entry);
+                }
             }
         }
 
+        ColorModel colorModel = imageType.getColorModel();
+        SampleModel sampleModel = imageType.getSampleModel();
+        int numBands = sampleModel.getNumBands();
+        int pixelSize = computePixelSize(sampleModel);
+
+        entries.put(TIFF.TAG_BITS_PER_SAMPLE, new TIFFEntry(TIFF.TAG_BITS_PER_SAMPLE, asShortArray(sampleModel.getSampleSize())));
+
+        // Compression field from param or metadata
         int compression;
         if ((param == null || param.getCompressionMode() == TIFFImageWriteParam.MODE_COPY_FROM_METADATA)
                 && ifd != null && ifd.getEntryById(TIFF.TAG_COMPRESSION) != null) {
@@ -936,11 +824,81 @@ public final class TIFFImageWriter extends ImageWriterBase {
         else {
             compression = TIFFImageWriteParam.getCompressionType(param);
         }
+        entries.put(TIFF.TAG_COMPRESSION, new TIFFEntry(TIFF.TAG_COMPRESSION, compression));
 
-        int photometricInterpretation = getPhotometricInterpretation(imageType.getColorModel(), compression);
+        // TODO: Allow metadata to take precedence?
+        int photometricInterpretation = getPhotometricInterpretation(colorModel, compression);
         entries.put(TIFF.TAG_PHOTOMETRIC_INTERPRETATION, new TIFFEntry(TIFF.TAG_PHOTOMETRIC_INTERPRETATION, TIFF.TYPE_SHORT, photometricInterpretation));
 
-        // TODO: Set values from param if != null + combined values...
+        // If numComponents > numColorComponents, write ExtraSamples
+        if (numBands > colorModel.getNumColorComponents()) {
+            // TODO: Write per component > numColorComponents
+            if (colorModel.hasAlpha()) {
+                entries.put(TIFF.TAG_EXTRA_SAMPLES, new TIFFEntry(TIFF.TAG_EXTRA_SAMPLES, colorModel.isAlphaPremultiplied() ? TIFFBaseline.EXTRASAMPLE_ASSOCIATED_ALPHA : TIFFBaseline.EXTRASAMPLE_UNASSOCIATED_ALPHA));
+            }
+            else {
+                entries.put(TIFF.TAG_EXTRA_SAMPLES, new TIFFEntry(TIFF.TAG_EXTRA_SAMPLES, TIFFBaseline.EXTRASAMPLE_UNSPECIFIED));
+            }
+        }
+
+        switch (compression) {
+            case TIFFExtension.COMPRESSION_ZLIB:
+            case TIFFExtension.COMPRESSION_DEFLATE:
+            case TIFFExtension.COMPRESSION_LZW:
+                // TODO: Let param/metadata control predictor
+                // TODO: Depending on param.getCompressionMode(): DISABLED/EXPLICIT/COPY_FROM_METADATA/DEFAULT
+                if (pixelSize >= 8) {
+                    entries.put(TIFF.TAG_PREDICTOR, new TIFFEntry(TIFF.TAG_PREDICTOR, TIFFExtension.PREDICTOR_HORIZONTAL_DIFFERENCING));
+                }
+
+                break;
+
+            case TIFFExtension.COMPRESSION_CCITT_T4:
+                Entry group3options = ifd != null ? ifd.getEntryById(TIFF.TAG_GROUP3OPTIONS) : null;
+
+                if (group3options == null) {
+                    group3options = new TIFFEntry(TIFF.TAG_GROUP3OPTIONS, (long) TIFFExtension.GROUP3OPT_2DENCODING);
+                }
+
+                entries.put(TIFF.TAG_GROUP3OPTIONS, group3options);
+
+                break;
+
+            case TIFFExtension.COMPRESSION_CCITT_T6:
+                Entry group4options = ifd != null ? ifd.getEntryById(TIFF.TAG_GROUP4OPTIONS) : null;
+
+                if (group4options == null) {
+                    group4options = new TIFFEntry(TIFF.TAG_GROUP4OPTIONS, 0L);
+                }
+
+                entries.put(TIFF.TAG_GROUP4OPTIONS, group4options);
+
+                break;
+
+            default:
+        }
+
+        if (photometricInterpretation == TIFFBaseline.PHOTOMETRIC_PALETTE && colorModel instanceof IndexColorModel) {
+            // TODO: Fix consistency between sampleModel.getSampleSize() and colorModel.getPixelSize()...
+            // We should be able to support 1, 2, 4 and 8 bits per sample at least, and probably 3, 5, 6 and 7 too
+            entries.put(TIFF.TAG_COLOR_MAP, new TIFFEntry(TIFF.TAG_COLOR_MAP, createColorMap((IndexColorModel) colorModel, sampleModel.getSampleSize(0))));
+            entries.put(TIFF.TAG_SAMPLES_PER_PIXEL, new TIFFEntry(TIFF.TAG_SAMPLES_PER_PIXEL, 1));
+        }
+        else {
+            entries.put(TIFF.TAG_SAMPLES_PER_PIXEL, new TIFFEntry(TIFF.TAG_SAMPLES_PER_PIXEL, numBands));
+
+            // Note: Assuming sRGB to be the default RGB interpretation
+            ColorSpace colorSpace = colorModel.getColorSpace();
+            if (colorSpace instanceof ICC_ColorSpace && !colorSpace.isCS_sRGB()) {
+                entries.put(TIFF.TAG_ICC_PROFILE, new TIFFEntry(TIFF.TAG_ICC_PROFILE, ((ICC_ColorSpace) colorSpace).getProfile().getData()));
+            }
+        }
+
+        // Default sample format SAMPLEFORMAT_UINT need not be written
+        if (sampleModel.getDataType() == DataBuffer.TYPE_SHORT/* TODO: if isSigned(sampleModel.getDataType) or getSampleFormat(sampleModel) != 0 */) {
+            entries.put(TIFF.TAG_SAMPLE_FORMAT, new TIFFEntry(TIFF.TAG_SAMPLE_FORMAT, TIFFExtension.SAMPLEFORMAT_INT));
+        }
+        // TODO: Float values!
 
         return new TIFFImageMetadata(entries.values());
     }
@@ -968,30 +926,32 @@ public final class TIFFImageWriter extends ImageWriterBase {
     }
 
     @Override
-    public void prepareWriteSequence(IIOMetadata streamMetadata) throws IOException {
+    public void prepareWriteSequence(final IIOMetadata streamMetadata) throws IOException {
         if (writingSequence) {
             throw new IllegalStateException("sequence writing has already been started!");
         }
 
         assertOutput();
+        configureStreamByteOrder(streamMetadata, imageOutput);
 
         writingSequence = true;
-        sequenceExifWriter = new EXIFWriter();
-        sequenceExifWriter.writeTIFFHeader(imageOutput);
+        sequenceTIFFWriter = new TIFFWriter();
+        sequenceTIFFWriter.writeTIFFHeader(imageOutput);
         sequenceLastIFDPos = imageOutput.getStreamPosition();
     }
 
     @Override
-    public void writeToSequence(IIOImage image, ImageWriteParam param) throws IOException {
+    public void writeToSequence(final IIOImage image, final ImageWriteParam param) throws IOException {
         if (!writingSequence) {
             throw new IllegalStateException("prepareWriteSequence() must be called before writeToSequence()!");
         }
 
-        if (sequenceLastIFDPos > 0) {
+        if (sequenceIndex > 0) {
             imageOutput.flushBefore(sequenceLastIFDPos);
+            imageOutput.seek(imageOutput.length());
         }
 
-        sequenceLastIFDPos = writePage(sequenceIndex++, image, param, sequenceExifWriter, sequenceLastIFDPos);
+        sequenceLastIFDPos = writePage(sequenceIndex++, image, param, sequenceTIFFWriter, sequenceLastIFDPos);
     }
 
     @Override
@@ -1002,7 +962,7 @@ public final class TIFFImageWriter extends ImageWriterBase {
 
         writingSequence = false;
         sequenceIndex = 0;
-        sequenceExifWriter = null;
+        sequenceTIFFWriter = null;
         sequenceLastIFDPos = -1;
         imageOutput.flush();
     }
@@ -1013,7 +973,7 @@ public final class TIFFImageWriter extends ImageWriterBase {
 
         writingSequence = false;
         sequenceIndex = 0;
-        sequenceExifWriter = null;
+        sequenceTIFFWriter = null;
         sequenceLastIFDPos = -1;
     }
 
